/*
  CDCVKBMS

  A simple example for Virtual Keyboad and mouse.
  CDC would forward the data it received to Keyboard or Mouse
  In this way we could emulated a Keyboard and Mouse

  created 2024
  by Deqing Sun for use with CH55xduino

  This example code is in the public domain.

  cli board options: usb_settings=user148

*/


#ifndef USER_USB_RAM
#error "This example needs to be compiled with a USER USB setting"
#endif

#include <WS2812.h>
#include "src/CdcHidCombo/USBCDC.h"
#include "src/CdcHidCombo/USBHIDKeyboardMouse.h"

#define NUM_LEDS 1
#define COLOR_PER_LEDS 3
#define NUM_BYTES (NUM_LEDS*COLOR_PER_LEDS)

__xdata uint8_t ledData[NUM_BYTES];

#define  KeyboardReportID  0x01
#define  rMouseReportID    0x02
#define  aMouseReportID    0x03
#define  OnBoardLED        0x04

// Data format
// Keyboard(Total 9 bytes)： 01(ReportID 01) + Keyboard data (8 Bytes)
// Mouse(Total 5 bytes)： 02(ReportID 02) + Mouse Data (4 Bytes)
uint8_t recvStr[9];
uint8_t recvStrPtr = 0;
unsigned long Elsp;

void setup() {
  USBInit();
  Serial0_begin(115200);
  delay(1000);
  Serial0_print("start");
  Elsp=0;
}

void loop() {
  while (USBSerial_available()) {
    uint8_t serialChar = USBSerial_read();
    recvStr[recvStrPtr++] = serialChar;
    if (recvStrPtr == 10) {
      for (uint8_t i = 0; i < 9; i++) {
        Serial0_write(recvStr[i]);
      }      
      if (recvStr[0] == KeyboardReportID) { // Keyboard
        USB_EP3_send(recvStr, 9);
      }
      if (recvStr[0] == rMouseReportID) {
        USB_EP3_send(recvStr, 5); // Relative Mouse
      }
      if (recvStr[0] == aMouseReportID) {
        USB_EP3_send(recvStr, 7); // Absolute Mouse
      }      
      if (recvStr[0] == OnBoardLED) {
        set_pixel_for_GRB_LED(ledData, 0, recvStr[1], recvStr[2], recvStr[3]); 
        neopixel_show_P1_5(ledData, NUM_BYTES);  
      }



      recvStrPtr = 0;
    }
    Elsp=millis();
  }
  // If there is no data in 100ms, clear the receive buffer
  if (millis()-Elsp>100) {
      recvStrPtr = 0;
      Elsp=millis();
    }
}
